﻿/**
 * Author:     Alberto Gil Tesa
 * WebSite:    http://giltesa.com
 * License:    CC BY-NC-SA 3.0
 *
 * ATTiny85 pinout:
 *                             _____
 *                     Rst A0 |1   8| VCC
 *     Tira de leds => D3  A3 |2   7| D2  A1
 *    Potenciometro => D4  A2 |3   6| D1  PWM
 *                        GND |4___5| D0  PWM
*/



// PINES
//#define LED  3   // Digital PWM.
//#define POT A2   // Analogico.

#define LED  12   // Digital PWM.
#define POT A0   // Analogico.



// BRILLO:
#define bMinimo   1  // Valor minimo para que el led este encendido.
#define bMargen   2  // Diferencia que ha de haber para que se considere que se ha cambiado el brillo.
byte bAnterior  = 0;
byte bActual    = getBrillo();


// MODO AHORRO:
#define tiempoApagado     10000  //60000 //3600000 // Tiempo para entrar en modo ahorro (apagado).
#define tiempoAtenuado    10     // Velocidad que se tarda en aumentar o disminuir el brillo.
unsigned long ultimoUso = 0;     // El momento en el que se utilizo por ultima vez el potenciometro.
boolean enAhorro        = false;



void setup()
{
	pinMode(LED, OUTPUT);
}



void loop()
{
	bActual = getBrillo();


	// El movimiento del potenciometro ha de superar el margen para que responda (esto previene ruido y comportamientos erraticos)
	if( getDiferenciaBrillo() > bMargen )
	{
		bAnterior = bActual;
		ultimoUso = millis();

		// Enciende el LED gradualmente:
		if( enAhorro )
		{
			setBrillo(true);
			enAhorro = false;
		}
		// Enciende el LED si supera el minimo de brillo:
		else if( bActual > bMinimo )
		{
			analogWrite( LED, bActual );
		}
		else
		{
			analogWrite( LED, 0 );
		}
	}


	// Activa el modo ahorro si el led esta encendido y ha pasado 10 segundos:
	if( !enAhorro  &&  bActual > bMinimo  &&  millis()-ultimoUso > tiempoApagado )
	{
		setBrillo(false);
		enAhorro = true;
	}

}



/**
 * Lee el valor del potenciometro y
 * devuelve el brillo correspondiente para el led.
 */
byte getBrillo()
{
	return map(analogRead(POT), 0, 1023, 0, 255);
}



/**
 * Enciende o apaga el LED con efecto gradual.
 */
void setBrillo( boolean estado )
{
	// Sube el brillo:
	if( estado )
	{
		bActual = getBrillo();

		for( int i=0 ; i<=bActual ; i++ )
		{
			analogWrite(LED, i);
			delay(tiempoAtenuado);
		}
	}
	// Baja el brillo:
	else
	{
		for( int i=bActual ; i>=0 ; i-- )
		{
			analogWrite(LED, i);
			delay(tiempoAtenuado);
		}
		bActual = 0;
	}
}



/**
 * Devuelve la diferencia de brillo entre la medicion
 * anterior y la actual.
 */
byte getDiferenciaBrillo()
{
	return bAnterior > bActual ? bAnterior-bActual : bActual-bAnterior;
}